package br.usp.poli.java.ihc;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import br.usp.poli.java.JogadorDePoquerFechado;
import br.usp.poli.java.JogadorDeTexasHoldem;

/**
 * Classe que cuida da Thread do Jogo. <br />
 * O uso de uma thread evita que problemas do jogo afetem a interface
 * grfica. Mais que isso, permite que o jogo seja interrompido, caso
 * a implementao do Jogo trave - ou se deseje parar o jogo por qualquer
 * motivo.
 * @author FLevy
 */
public class ThreadPoquer {
	private static final int TEMPO_ESPERA = 500;
	private JogoDeTexasHoldem jogoTexas;
	private JogoDePoquerFechado jogoFechado;
	private ThreadInterna atual;
	private Thread threadAtiva;
	
	/**
	 * Cria um jogo de poquer a partir de duas implementaes de JogoDePoquer,
	 * uma para Texas Hold'em e outra para Poquer Fechado.<br>
	 * 
	 * @param jogoTexas Instncia do jogo de Texas Hold'em.
	 * @param jogoFechado Instncia do jogo de Poquer Fechado.
	 */
	public ThreadPoquer (JogoDeTexasHoldem jogoTexas, JogoDePoquerFechado jogoFechado) {
		this.jogoTexas = jogoTexas;
		this.jogoFechado = jogoFechado;
	}
	
	/**
	 * Inicia um jogo de Poquer de Poquer Fechado.
	 * @param j1 O primeiro jogador.
	 * @param j2 O segundo jogador.
	 */
	public void iniciar(JogadorDePoquerFechadoIHC j1, JogadorDePoquerFechadoIHC j2) {
		atual = new ThreadInterna(jogoFechado, j1, j2);
		threadAtiva = new Thread(atual);
		threadAtiva.start();
	}
	
	/**
	 * Inicia um jogo de Poquer de Texas Hold'em.
	 * @param j1 O primeiro jogador.
	 * @param j2 O segundo jogador.
	 */
	public void iniciar(JogadorDeTexasHoldemIHC j1, JogadorDeTexasHoldemIHC j2) {
		atual = new ThreadInterna(jogoTexas, j1, j2);
		threadAtiva = new Thread(atual);
		threadAtiva.start();
	}
	
	/**
	 * Interrompe o jogo iniciado.
	 */
	public void interromper() {
		atual.interromper();
		while (threadAtiva.isAlive()) {
			try {
				Thread.sleep(TEMPO_ESPERA);
			} catch (InterruptedException e) {
				// no devia acontecer, mas vai saber...
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Classe que cria uma Thread para executar o jogo separadamente.
	 * Assim se evita problemas de interao entre a interface grfica e
	 * o jogo em si.
	 * @author FLevy
	 */
	private static class ThreadInterna implements Runnable {
		private Jogo jogo;
		private JogadorIHC j1, j2;
		
		/**
		 * Cria uma Thread para um jogo de texas hold'em.
		 * @param jogo O jogo de texas hold'em.
		 * @param j1 O jogador 1.
		 * @param j2 O jogador 2.
		 */
		ThreadInterna(JogoDeTexasHoldem jogo, JogadorDeTexasHoldemIHC j1, JogadorDeTexasHoldemIHC j2) {
			this.jogo = jogo;
			this.j1 = j1;
			this.j2 = j2;
		}
		
		/**
		 * Cria uma Thread para um jogo de poquer fechado.
		 * @param jogo O jogo de poquer fechado.
		 * @param j1 O jogador 1.
		 * @param j2 O jogador 2.
		 */		
		ThreadInterna(JogoDePoquerFechado jogo, JogadorDePoquerFechadoIHC j1, JogadorDePoquerFechadoIHC j2) {
			this.jogo = jogo;
			this.j1 = j1;
			this.j2 = j2;
		}
		
		/**
		 * Executa o jogo, at que ele termine com um vencedor ou at que ele
		 * seja interrompido.
		 */
		public void run() {
			try {
				JogadorIHC vencedor;
				
				if (jogo instanceof JogoDeTexasHoldem) {
					vencedor = (JogadorIHC) ((JogoDeTexasHoldem) jogo).iniciar((JogadorDeTexasHoldem) j1, (JogadorDeTexasHoldem) j2);
				} else {
					vencedor = (JogadorIHC) ((JogoDePoquerFechado) jogo).iniciar((JogadorDePoquerFechado) j1, (JogadorDePoquerFechado) j2);
				}
				
				j1.avisarFimDoJogo(vencedor == j1);
				j2.avisarFimDoJogo(vencedor == j2);
			} catch (JogoInterrompido e) {
				// jogo interrompido!
				return;
			} catch (Exception e) {
				// No devia acontecer
				e.printStackTrace();
			}
		}
		
		/**
		 * Pede para interromper o jogo.
		 */
		public void interromper() {
			j1.interromper();
			j2.interromper();
		}
	}
}
